// Pins:
// 0,1 RX/TX
// 2,3 NC... add more colors here :-)
int inverter[] = {4, 5};
int button     = 7;
int leds[]     = {8, 9, 10, 11, 12, 13};
int light[]    = {18, 14}; //{sensor, limit}
int temp[]     = {17, 15};
int bounce[]   = {6, 16};
// 19 -> 5V for light sensor (you don't need that)
// (stupid, but i changed the sensor after designing the shield :-)

// offsets for hysteresis
int offsetLight  = 20;
int offsetTemp   = 50;
int offsetBounce = 50;

// naming convention for switching between colors
boolean red  = true;
boolean blue = false;

int baseDelay = 500;
int mode = 0;

// store analog input values
int sensorValue;
int resistorValue;

void setup(void) {
  Serial.begin(9600); 

  pinMode(inverter[0], OUTPUT);
  pinMode(inverter[1], OUTPUT);
  pinMode(bounce[0], INPUT_PULLUP);
  pinMode(button, INPUT_PULLUP);
  for(int i=0; i<6; i++) {
    pinMode(leds[i], OUTPUT);
    digitalWrite(leds[i], LOW); // all led off
  }
  //pinMode(19, OUTPUT); // just used as an additional 5V supply
  //digitalWrite(19, HIGH); // 5V output for light sensor
  
  digitalWrite(leds[mode], HIGH); // one led on for the startmode
  allOn(false); // inverter off
}

void loop(void) {
  // test button, and chane mode
  if (!digitalRead(button)) {
    digitalWrite(leds[mode], LOW); // old off
    mode = (mode+1)%6;
    digitalWrite(leds[mode], HIGH); // new on
  }

  if (mode == 0) { // Low light level turns light on
    checkSensor(light);
    if (sensorValue < resistorValue) { // lights on
      Serial.println(" -> Dark");
      allOn(true);
    } 
    else if (sensorValue > resistorValue + offsetLight) { // lights off
      Serial.println(" -> Bright");
      allOn(false);
    }
  } 
  else if (mode == 1) { //Color corresponding to temperature
    checkSensor(temp);
    if (sensorValue < resistorValue) { // blue light
      Serial.println("Cold");
      oneOn(blue);
    } 
    else if (sensorValue > resistorValue + offsetTemp) { // red light
      Serial.println("Hot");
      oneOn(red);
    }
  } 
  else if (mode == 2) { // color dependent on positon, with blinking
    checkSensor(bounce);
    if (resistorValue > offsetBounce) {
      allOn(false);
      delay(resistorValue);
    }
    if (digitalRead(bounce[0])) {
      Serial.println("Blue");
      oneOn(blue);
    } 
    else {
      Serial.println("Red");
      oneOn(red);
    } 
  } 
  else if (mode == 3) { // just the blue light is on
    Serial.println("Blue light");
    oneOn(blue); 
  } 
  else if (mode == 4) { // just the red light is on
    Serial.println("Red light");
    oneOn(red);
  } 
  else if (mode == 5) { // all lights are on
    Serial.println("Red and Blue");
    allOn(true);
  } else { // impossible to reach :-)
    Serial.println("Congratulation, you have reached the unreachable Mode!");
  }
  delay(baseDelay);
}

void allOn(boolean on) { // all inverter on, iff not already on
  if (digitalRead(inverter[0]) != on) digitalWrite(inverter[0], on);
  if (digitalRead(inverter[1]) != on) digitalWrite(inverter[1], on);
}

void oneOn(boolean color) { // one inverter on, the other off, iff not already done
  if (digitalRead(inverter[color]) == true) digitalWrite(inverter[color], false);
  if (digitalRead(inverter[!color]) == false) digitalWrite(inverter[!color], true);
}

void checkSensor(int sensor[]) { // read input and store/transmit them
  sensorValue = analogRead(sensor[0]);
  resistorValue = analogRead(sensor[1]);
  Serial.print("Sensor: ");
  Serial.print(sensorValue);
  Serial.print(", Poti: ");
  Serial.println(resistorValue);
}

